package org.iotivity.bridge.homeserver.handler.client;

import android.util.Log;
import android.util.Pair;

import org.iotivity.OCArray;
import org.iotivity.OCClientResponse;
import org.iotivity.OCInterfaceMask;
import org.iotivity.OCRep;
import org.iotivity.OCRepresentation;
import org.iotivity.OCResponseHandler;
import org.iotivity.OCStatus;
import org.iotivity.bridge.homeserver.BridgeServerActivity;
import org.iotivity.bridge.homeserver.utils.Utils;
import org.iotivity.oc.OcRemoteResource;

import java.util.ArrayList;
import java.util.Arrays;

public class GetCommonResponseHandler implements OCResponseHandler {

    private static final String TAG = GetCommonResponseHandler.class.getSimpleName();

    private BridgeServerActivity activity;
    private OcRemoteResource resource ;

    public GetCommonResponseHandler(BridgeServerActivity activity, OcRemoteResource resource) {
        this.activity = activity ;
        this.resource = resource ;
    }

    @Override
    public void handler(OCClientResponse response) {
        activity.msg("Get "+resource.getUri()+" Response Handler: Code = "+response.getCode().toString());
        if(response.getCode() != OCStatus.OC_STATUS_OK) {
            activity.showErrorDialog("Get "+resource.getUri()+" Response Handler Error",
                    "Result Code = " + response.getCode().toString());
            activity.printLine();
            return;
        }

        final ArrayList<Pair<String, Object>> props = new ArrayList<>();
        OCRepresentation rep = response.getPayload();
        while (rep != null) {
            activity.msg("\t" + "Type" + " = " + rep.getType().name());
            switch (rep.getType()) {
                case OC_REP_BOOL:
                    activity.msg("\t" + rep.getName() + " = " + rep.getValue().getBool());
                    props.add(new Pair(rep.getName(), rep.getValue().getBool()));
                    break;
                case OC_REP_INT:
                    activity.msg("\t" + rep.getName() + " = " + rep.getValue().getInteger());
                    props.add(new Pair(rep.getName(), rep.getValue().getInteger()));
                    break;
                case OC_REP_STRING:
                    activity.msg("\t" + rep.getName() + " = " + rep.getValue().getString());
                    props.add(new Pair(rep.getName(), rep.getValue().getString()));
                    break;
                case OC_REP_DOUBLE:
                    activity.msg("\t" + rep.getName() + " = " + rep.getValue().getDouble());
                    props.add(new Pair(rep.getName(), rep.getValue().getDouble()));
                    break;
                case OC_REP_STRING_ARRAY:
                    String[] str_arr = OCRep.ocArrayToStringArray(rep.getValue().getArray());
                    String str_strs = Utils.toString(str_arr);
                    activity.msg("\t" + rep.getName() + " = " + str_strs);
                    props.add(new Pair(rep.getName(), str_strs));
                    break ;
                case OC_REP_BOOL_ARRAY:
                    boolean[] bool_arr = OCRep.ocArrayToBooleanArray(rep.getValue().getArray());
                    String bool_strs = Utils.toString(bool_arr);
                    activity.msg("\t" + rep.getName() + " = " + bool_strs);
                    props.add(new Pair(rep.getName(), bool_strs));
                    break;
                case OC_REP_INT_ARRAY:
                    long[] long_arr = OCRep.ocArrayToLongArray(rep.getValue().getArray());
                    String long_strs = Utils.toString(long_arr);
                    activity.msg("\t" + rep.getName() + " = " + long_strs);
                    props.add(new Pair(rep.getName(), long_strs));
                    break;
                case OC_REP_DOUBLE_ARRAY:
                    double[] double_arr = OCRep.ocArrayToDoubleArray(rep.getValue().getArray());
                    String double_strs = Utils.toString(double_arr);
                    activity.msg("\t" + rep.getName() + " = " + double_strs);
                    props.add(new Pair(rep.getName(), double_strs));
                    break;
                case OC_REP_OBJECT:
                    {
                        OCRepresentation object = rep.getValue().getObject();
                        StringBuffer sb = new StringBuffer();
                        sb.append("{");
                        while (object != null) {
                            switch (object.getType()) {
                                case OC_REP_STRING:
                                    sb.append(object.getName() + " = " + object.getValue().getString());
                                    break;
                                default:
                                    break;
                            }
                            object = object.getNext();
                            if (object != null) sb.append(", ");
                        }
                        sb.append("}");
                        activity.msg("\t" + rep.getName() + " = " + sb.toString());
                        props.add(new Pair(rep.getName(), sb.toString()));
                    }
                    break;
                case OC_REP_OBJECT_ARRAY:
                    {
                        OCRepresentation array = rep.getValue().getObject();
                        StringBuffer sb = new StringBuffer();
                        sb.append("[");
                        while (array != null) {
                            activity.msg("\t" + "OC_REP_OBJECT_ARRAY Type" + " = " + array.getType().name());
                            switch (array.getType()) {
                                case OC_REP_OBJECT: {
                                    OCRepresentation object = array.getValue().getObject();
                                    sb.append("{");
                                    while (object != null) {
                                        activity.msg("\t" + "OC_REP_OBJECT Type" + " = " + object.getType().name());
                                        switch (object.getType()) {
                                            case OC_REP_STRING:
                                                sb.append(object.getName() + " = " + object.getValue().getString());
                                                break;
                                            case OC_REP_BOOL:
                                                sb.append(object.getName() + " = " + object.getValue().getBool());
                                                break;
                                            case OC_REP_INT:
                                                sb.append(object.getName() + " = " + object.getValue().getInteger());
                                                break;
                                            case OC_REP_DOUBLE:
                                                sb.append(object.getName() + " = " + object.getValue().getDouble());
                                                break;
                                            case OC_REP_STRING_ARRAY:
                                                activity.msg("\tarray size = " + OCRep.ocArrayToStringArray(rep.getValue().getArray()).length);

                                                sb.append(object.getName() + " = " + Arrays.toString(OCRep.ocArrayToStringArray(rep.getValue().getArray())));
                                                break;
                                            case OC_REP_INT_ARRAY:
                                                sb.append(object.getName() + " = " + Arrays.toString(OCRep.ocArrayToLongArray(rep.getValue().getArray())));
                                                break;
                                            case OC_REP_DOUBLE_ARRAY:
                                                sb.append(object.getName() + " = " + Arrays.toString(OCRep.ocArrayToDoubleArray(rep.getValue().getArray())));
                                                break;
                                            case OC_REP_BOOL_ARRAY:
                                                sb.append(object.getName() + " = " + Arrays.toString(OCRep.ocArrayToBooleanArray(rep.getValue().getArray())));
                                                break;
                                            default:
                                                break;
                                        }
                                        object = object.getNext();
                                        if (object != null) sb.append(", ");
                                    }
                                    sb.append("}");
                                }
                                    break;
                                default:
                                    break;
                            }
                            array = array.getNext();
                            if (array != null) sb.append(", ");
                        }
                        sb.append("]");
                        activity.msg("\t" + rep.getName() + " = " + sb.toString());
                        props.add(new Pair(rep.getName(), sb.toString()));
                    }
                    break;
                default:
                    break;
            }
            rep = rep.getNext();
        }
        activity.printLine();

        if(resource.getInterfaceMask() == OCInterfaceMask.RW ||
                resource.getInterfaceMask() == OCInterfaceMask.A) {
            activity.showPropertiesDialog( "Properties of " + this.resource.getUri(), props, resource);
            Log.d(TAG, "Properties : " + Utils.propertiesToString(props));
        } else {
            activity.showResultDialog("Get " + resource.getUri() + " Response Handler Result",
                    "Result Code = " + response.getCode().toString() +"\n\n"+ Utils.propertiesToString(props));
            Log.d(TAG, "Properties : " + Utils.propertiesToString(props));
        }
    }
}
