package org.iotivity.bridge.homeserver.utils;

import android.content.Context;
import android.util.Log;
import android.util.Pair;
import android.webkit.DownloadListener;
import android.widget.Switch;

import org.iotivity.OCEndpoint;
import org.iotivity.bridge.homeserver.handler.FactoryPresetsHandler;
import org.iotivity.oc.OcRemoteDevice;
import org.iotivity.oc.OcRemoteResource;
import org.iotivity.oc.OcResource;
import org.iotivity.oc.OcUtils;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;

public class Utils {

    private static final String TAG = Utils.class.getSimpleName();

    public static boolean containsList(String[] arr, String targetValue) {
        return Arrays.asList(arr).contains(targetValue);
    }

    public static boolean containsSet(String[] arr, String targetValue) {
        Set<String> set = new HashSet<String>(Arrays.asList(arr));
        return set.contains(targetValue);
    }

    public static boolean containsLoop(String[] arr, String targetValue) {
        for(String s: arr){
            if(s.equals(targetValue))
                return true;
        }
        return false;
    }

    public static boolean containsArraysBinarySearch(String[] arr, String targetValue) {
        Arrays.sort(arr);
        int a =  Arrays.binarySearch(arr, targetValue);
        if(a > 0)
            return true;
        else
            return false;
    }

    public static OCEndpoint containsEndPointList(OCEndpoint[] arr, String startValue) {
        for(OCEndpoint ep: arr){
            if(OcUtils.endpointToString(ep).contains(startValue))
                return ep;
        }
        return null;
    }

    public static OCEndpoint getCoapsEndpoint (OCEndpoint[] arr) {
        OCEndpoint endpoint = Utils.containsEndPointList(arr, "coaps");
        if (endpoint != null) {
            return endpoint ;
        } else {
            return arr[0];
        }
    }

    public static OCEndpoint getCoapEndpoint (OCEndpoint[] arr) {
        OCEndpoint endpoint = Utils.containsEndPointList(arr, "coap");
        if (endpoint != null) {
            return endpoint ;
        } else {
            return arr[0];
        }
    }

    public static boolean isRemoteDCRs(OcRemoteResource resource) {
        return isDCRs(resource.getTypes());
    }

    public static boolean isOwnDCRs(OcResource resource) {
        return isDCRs(resource.getResourceTypes());
    }

    public static boolean isDCRs(String[] resource_types) {   // Device Configuration Resource

        // Discovery Core Resources
        if(containsList(resource_types, "oic.wk.res")          // Implicit Invisible Discovery Resource in client discovery
                || containsList(resource_types, "oic.wk.d")    // Device Resource
                || containsList(resource_types, "oic.wk.p")) { // Platform Resource
            return true ;
        }
        // Optional Core Resource
        else if(containsList(resource_types, "oic.wk.con")     // Configuration Resource
                || containsList(resource_types, "oic.wk.introspection") // Device Introspection
                || containsList(resource_types, "x.org.openconnectivity.oic.introspection.data")) {   // Device Introspection Data
            return false ;
        }
        // Security Virtual Resources
        else if (containsList(resource_types, "oic.r.doxm")            // Device Ownership Transfer Method Resource
                || containsList(resource_types, "oic.r.pstat")         // Provision State Resource
                || containsList(resource_types, "oic.r.acl2")          // Access Control List Resource
                || containsList(resource_types, "oic.r.cred")          // Credentials Resource
                || containsList(resource_types, "oic.r.sp")            // Security Profile Resource
                || containsList(resource_types, "oic.r.csr")           // Certificate Signing Request Resource
                || containsList(resource_types, "oic.r.roles")) {      // Role Maintain Resource with Role certificate
            return true ;
        }
        else if (containsList(resource_types, "oic.wk.mnt")                // Management Resource
                || containsList(resource_types, "oic.wk.nmon")             // Network Monitoring Resource
                || containsList(resource_types, "oic.r.softwareupdate")    // Software Update Resource
                || containsList(resource_types, "oic.r.coapcloudconf")) {  // Cloud Resource for CoapCloudConf
            return true ;
        }
        return false ;
    }

    /* Non-Configuration Resource with auth-crypt wildcard(*) permission
    * and Core Resource with anon-clear /oic/res(oic.wk.res), /oic/d(oic.wk.d), /oic/p(oic.wk.p) permission
    * in ACL2 (/oic/sec/acl2)
    */
    public static boolean isAccessableByPairwisedClient(String[] resource_types) {
        // Discovery Core Resources
        if(containsList(resource_types, "oic.wk.res")          // Implicit Invisible Discovery Resource in client discovery
                || containsList(resource_types, "oic.wk.d")    // Device Resource
                || containsList(resource_types, "oic.wk.p")) { // Platform Resource
            return true ;
        }
        // Optional Core Resource
        else if(containsList(resource_types, "oic.wk.con")     // Configuration Resource
                || containsList(resource_types, "oic.wk.introspection") // Device Introspection
                || containsList(resource_types, "x.org.openconnectivity.oic.introspection.data")) {   // Device Introspection Data
            return true ;
        }
        // Security Virtual Resources
        else if (containsList(resource_types, "oic.r.doxm")            // Device Ownership Transfer Method Resource
                || containsList(resource_types, "oic.r.pstat")         // Provision State Resource
                || containsList(resource_types, "oic.r.acl2")          // Access Control List Resource
                || containsList(resource_types, "oic.r.cred")          // Credentials Resource
                || containsList(resource_types, "oic.r.sp")            // Security Profile Resource
                || containsList(resource_types, "oic.r.csr")           // Certificate Signing Request Resource
                || containsList(resource_types, "oic.r.roles")) {      // Role Maintain Resource with Role certificate
            return false ;
        }
        else if (containsList(resource_types, "oic.wk.mnt")                // Management Resource
                || containsList(resource_types, "oic.wk.nmon")             // Network Monitoring Resource
                || containsList(resource_types, "oic.r.softwareupdate")    // Software Update Resource
                || containsList(resource_types, "oic.r.coapcloudconf")) {  // Cloud Resource for CoapCloudConf
            return false ;
        }
        return true ;
    }

    public static boolean isObt(OcRemoteDevice device) {

        return containsDeviceType(device, "oic.d.dots") ;
    }
    public static boolean isBridge(OcRemoteDevice device) {

        return containsDeviceType(device, "oic.d.bridge") ;
    }

    public static boolean isVod(OcRemoteDevice device) {

        return containsDeviceType(device, "oic.d.virtual") ;
    }

    public static boolean containsDeviceType(OcRemoteDevice device, String device_type) {

        for(OcRemoteResource resource : device.getResources()) {
            if(containsList(resource.getTypes(), device_type)) {
                return true ;
            }
        }
        return false ;
    }

    public static String propertiesToString(ArrayList<Pair<String, Object>> props) {

        StringBuffer sb = new StringBuffer();
        for(Pair prop : props) {
            sb.append(prop.first + " = " + prop.second.toString() + "\n");
        }
        return sb.toString();
    }

    public static String toString(String[] arr) {
        StringBuffer sb = new StringBuffer();
        sb.append("[");
        for(int index = 0; index < arr.length; index++) {
            String obj = arr[index];
            sb.append(obj + ( index < arr.length -1 ? ", " : ""));
        }
        sb.append("]");
        return sb.toString();
    }

    public static String toString(long[] arr) {
        StringBuffer sb = new StringBuffer();
        sb.append("[");
        for(int index = 0; index < arr.length; index++) {
            Long obj = arr[index];
            sb.append(obj + ( index < arr.length -1 ? ", " : ""));
        }
        sb.append("]");
        return sb.toString();
    }

    public static String toString(double[] arr) {
        StringBuffer sb = new StringBuffer();
        sb.append("[");
        for(int index = 0; index < arr.length; index++) {
            Double obj = arr[index];
            sb.append(obj + ( index < arr.length -1 ? ", " : ""));
        }
        sb.append("]");
        return sb.toString();
    }

    public static String toString(boolean[] arr) {
        StringBuffer sb = new StringBuffer();
        sb.append("[");
        for(int index = 0; index < arr.length; index++) {
            boolean obj = arr[index];
            sb.append(obj + ( index < arr.length -1 ? ", " : ""));
        }
        sb.append("]");
        return sb.toString();
    }
}
